Package: retrosheet
Type: Package
Title: Import Professional Baseball Data from 'Retrosheet'
Version: 1.1.7
Authors@R: 
  c(person(given = "Colin",
           family = "Douglas",
           role = c("aut", "cre", "cph"),
           email = "retrosheet@colindougl.as",
           comment = c(ORCID = "https://orcid.org/0000-0002-6639-006X")),
    person(given = "Richard",
           family = "Scriven",
           role = c("aut", "cph"),
           email = "rmscriven@gmail.com"))
Date: 2026-04-18
Encoding: UTF-8
Maintainer: Colin Douglas <retrosheet@colindougl.as>
Description: A collection of tools to import and structure the (currently) single-season
    event, game-log, roster, and schedule data available from <https://www.retrosheet.org>.
    In particular, the event (a.k.a. play-by-play) files can be especially difficult to parse.
    This package does the parsing on those files, returning the requested data in the most
    practical R structure to use for sabermetric or other analyses.
URL: https://github.com/colindouglas/retrosheet
Depends: R (>= 2.10)
License: GPL (>= 2)
Imports: xml2 (>= 1.2.2), stringi (>= 0.4-1), httr (>= 1.4.1), stringr
        (>= 1.4.0), rvest (>= 0.3.5)
Note: NOTICE regarding the transfer of data from Retrosheet: The
        information used here was obtained free of charge from and is
        copyrighted by Retrosheet. Interested parties may contact
        Retrosheet at "www.retrosheet.org".
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), rmarkdown (>= 2.0.0)
BugReports: https://github.com/colindouglas/retrosheet/issues
NeedsCompilation: no
Packaged: 2026-04-20 11:19:53 UTC; colin
Author: Colin Douglas [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6639-006X>),
  Richard Scriven [aut, cph]
Repository: CRAN
Date/Publication: 2026-04-20 13:00:15 UTC
Built: R 4.5.2; ; 2026-04-20 14:45:54 UTC; unix

Package: retrosheet
Type: Package
Title: Import Professional Baseball Data from 'Retrosheet'
Version: 1.1.7
Authors@R: 
  c(person(given = "Colin",
           family = "Douglas",
           role = c("aut", "cre", "cph"),
           email = "retrosheet@colindougl.as",
           comment = c(ORCID = "https://orcid.org/0000-0002-6639-006X")),
    person(given = "Richard",
           family = "Scriven",
           role = c("aut", "cph"),
           email = "rmscriven@gmail.com"))
Date: 2026-04-18
Encoding: UTF-8
Maintainer: Colin Douglas <retrosheet@colindougl.as>
Description: A collection of tools to import and structure the (currently) single-season
    event, game-log, roster, and schedule data available from <https://www.retrosheet.org>.
    In particular, the event (a.k.a. play-by-play) files can be especially difficult to parse.
    This package does the parsing on those files, returning the requested data in the most
    practical R structure to use for sabermetric or other analyses.
URL: https://github.com/colindouglas/retrosheet
Depends: R (>= 2.10)
License: GPL (>= 2)
Imports: xml2 (>= 1.2.2), stringi (>= 0.4-1), httr (>= 1.4.1), stringr
        (>= 1.4.0), rvest (>= 0.3.5)
Note: NOTICE regarding the transfer of data from Retrosheet: The
        information used here was obtained free of charge from and is
        copyrighted by Retrosheet. Interested parties may contact
        Retrosheet at "www.retrosheet.org".
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), rmarkdown (>= 2.0.0)
BugReports: https://github.com/colindouglas/retrosheet/issues
NeedsCompilation: no
Packaged: 2026-04-20 11:19:53 UTC; colin
Author: Colin Douglas [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6639-006X>),
  Richard Scriven [aut, cph]
Repository: CRAN
Date/Publication: 2026-04-20 13:00:15 UTC
Built: R 4.6.0; ; 2026-04-26 09:56:42 UTC; unix

# Generated by roxygen2: do not edit by hand

export(gamelogFields)
export(getFileNames)
export(getParkIDs)
export(getPartialGamelog)
export(getRetrosheet)
export(getTeamIDs)
export(get_retrosheet)
importFrom(httr,RETRY)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(httr,timeout)
importFrom(httr,write_disk)
importFrom(rvest,html_attr)
importFrom(rvest,html_nodes)
importFrom(stats,setNames)
importFrom(stringi,stri_split_fixed)
importFrom(stringr,str_extract)
importFrom(utils,read.csv)
importFrom(utils,type.convert)
importFrom(utils,unzip)
importFrom(xml2,read_html)

# retrosheet

<!-- badges: start -->

[![R-CMD-check](https://github.com/colindouglas/retrosheet/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/colindouglas/retrosheet/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/retrosheet)](https://cran.r-project.org/package=retrosheet)
[![License: GPL
2.0](https://img.shields.io/badge/license-GPL%20(%3E%3D%202)-blue.svg)](https://opensource.org/license/gpl-2-0)
[![GitHub
commit](https://img.shields.io/github/last-commit/colindouglas/retrosheet)](https://github.com/colindouglas/retrosheet/commit/main)
[![Downloads](https://cranlogs.r-pkg.org/badges/last-week/retrosheet)](https://cran.r-project.org/package=retrosheet)
<!-- badges: end --> Import Retrosheet data as a structured R object

`retrosheet` is an R package that downloads and parses the single-season
event, gamelog, roster, and schedule files from
[retrosheet.org](https://www.retrosheet.org/) into structured R objects
for further analysis.

**Note:** As of retrosheet 1.1.0, this repo contains the version
available on CRAN. Older versions can be found
[here](https://github.com/rmscriven/retrosheet).

## Installation

`retrosheet` can be installed from CRAN, or development versions
installed from Github.

``` r
# Install from CRAN
install.packages("retrosheet")

# Install development version from Github
# install.packages("devtools")
devtools::install_github("colindouglas/retrosheet")
```

## Usage

- `getRetrosheet()` - This workhorse function returns the full seasonal
  data associated with the user-entered arguments
  - Use the `cache` argument to save a local cache and avoid stressing
    retrosheet.org
  - Use `get_retrosheet()` as a drop-in replacement to return tibbles
    instead of matrices
- `getPartialGamelog()` - An alternative to returning the full gamelog
  files. This function allows the user to choose the columns and date.
  Column names are made available by the global object `gamelogFields`

Also included are convenience functions

- `getFileNames()` - for obtaining a list of all zip files currently
  available for use by this package
- `getTeamIDs()` - for providing the team ID value to be used in the
  `team` argument of `getRetrosheet()`
- `getParkIDs()` - for ballpark ID and name information

#'
#' Import single-season retrosheet data as data frames
#'
#' This function is a wrapper for getRetrosheet(). It downloads and parses data from
#' \url{https://www.retrosheet.org} for the game-log, event, (play-by-play), roster, and schedule files.
#' While getRetrosheet() returns a list of matrices, this function returns an equivalent list of dataframes.
#' It takes the same arguments, and can act as a drop-in replacement.
#'
#' @param ... Arguments passed to `getRetrosheet()`. `stringsAsFactors` argument is always FALSE, and will warn if passed as TRUE
#'
#' @return The following return values are possible for the given \code{type}
#' \itemize{
#' \item \code{game} - a data frame of gamelog data for the given year
#' \item \code{play} - a list, each element of which is a single game's play-by-play
#' data for the given team and year.  Each list element is also a list, containing
#' the play-by-play data split into individual matrices.
#' \item \code{roster} - a named list, each element containing the roster
#' for the named team for the given year, as a data frame.
#' \item \code{schedule} - a data frame containing the game schedule for the given year
#' }
#'
#' @examples
#' \donttest{
#' ## get the full 1995 season schedule
#' get_retrosheet("schedule", 1995)
#'
#' ## get the same schedule, split by time of day
#' get_retrosheet("schedule", 1995, schedSplit = "TimeOfDay")
#'
#' ## get the roster data for the 1995 season, listed by team
#' get_retrosheet("roster", 1995)
#'
#' ## get the full gamelog data for the 2012 season
#' get_retrosheet("game", 2012)
#'
#' ## get the play-by-play data for the San Francisco Giants' 2012 season
#' get_retrosheet("play", 2012, "SFN")
#' }
#'
#' @importFrom utils type.convert
#' @export

get_retrosheet <- function(...) {

    # Warn if the function is called with stringsAsFactors = TRUE
    dots <- list(...)
    if (isTRUE(dots[["stringsAsFactors"]])) {
        warning("get_retrosheet() yields unexpected results with stringsAsFactors = TRUE. For more predictable behavior, use getRetroshet()")
        stringsAsFactors <- TRUE
    } else {
        stringsAsFactors <- FALSE
    }

    # Call the original function with all of the same arguments
    response <- getRetrosheet(...)

    matrix_to_df <- function(x) {

        # If the response is a single matrix, convert it to a tibble
        if (is.matrix(x)) x <- as.data.frame(x, stringsAsFactors = stringsAsFactors)

        # If the response is a dataframe (or was a matrix and has been converted to a dataframe)
        if (is.data.frame(x)) {

            # Aggressively guess the data type via type.convert
            out <- lapply(x, type.convert, na.strings = c("", "NA"), as.is = TRUE)

            # If there's a date column, make sure it's Date-type
            if ("Date" %in% names(out)) {
                out$Date <- as.character(out$Date)
                out$Date <- as.Date(out$Date, tryFormats = c("%Y-%m-%d", "%Y%m%d"))
            }

            return(as.data.frame(out, stringsAsFactors = stringsAsFactors))

        # If the response is a list of objects, re-apply this function to all of the objects in the list
        } else if (length(x) > 1) {
            lapply(x, matrix_to_df)
        } else {
            return(x) # If the response is not a list, matrix, or df, return the input
        }
    }

    matrix_to_df(response)

}

test_that("Caching works", {

    skip_if_offline(host = "retrosheet.org")

    # With caching
    schedule_1 <- get_retrosheet("schedule", 1995, cache = "testdata")
    schedule_1a <- get_retrosheet("schedule", 1995, cache = "testdata/") # Test with trailing slash
    roster_1 <- get_retrosheet("roster", 1995, cache = "testdata")
    game_1 <- get_retrosheet("game", 2012, cache = "testdata")
    play_1 <- get_retrosheet("play", 2012, "SFN", cache = "testdata")

    # Without caching
    schedule_2 <- get_retrosheet("schedule", 1995)
    roster_2 <- get_retrosheet("roster", 1995)
    game_2 <- get_retrosheet("game", 2012)
    play_2 <- get_retrosheet("play", 2012, "SFN")

    # With and without caching should always give you the same thing
    expect_equal(schedule_1, schedule_2)
    expect_equal(schedule_1, schedule_1a)
    expect_equal(roster_1, roster_2)
    expect_equal(game_1, game_2)
    expect_equal(play_1, play_2)

    # Re-using cached data should give a message about using a local cache
    expect_message(get_retrosheet("schedule", 1995, cache = "testdata"), "Using local cache: testdata/schedule/1995SKED.zip")

})

test_that("Schedule downloading works", {

    skip_if_offline(host = "retrosheet.org")

    schedule <- get_retrosheet(type = "schedule", year = 1995, cache = "testdata")
    schedule_unnamed <- get_retrosheet("schedule", 1995, cache = "testdata")
    schedule_splits <- get_retrosheet(type = "schedule", year = 1995, schedSplit = "TimeOfDay")
    schedule_splits_some_named <- get_retrosheet("schedule", 1995, schedSplit = "TimeOfDay")

    # In 1995, each of the 28 teams played 72 games
    expect_equal(nrow(schedule), 28 * 72)

    # Schedules should be the same, regardless of whether we used named params
    expect_equal(schedule_splits, schedule_splits_some_named)

    # There are three different splits, (D)ay, (E)vening, and (Night)
    expect_equal(length(schedule_splits), 3)

    # In 1995, there were 1355 (N)ight games
    expect_equal(nrow(schedule_splits$N), 1355)

    # There should be the same number of games in the split and unsplit schedules
    expect_equal(sum(unlist(lapply(schedule_splits, nrow), recursive = TRUE)), nrow(schedule))

    # Confirm that using named and unnamed arguments returns the same thing
    expect_equal(schedule, schedule_unnamed)

})

test_that("Roster downloading works", {

    skip_if_offline(host = "retrosheet.org")

    roster <- get_retrosheet("roster", 1995, cache = "testdata")

    # In 1995, there should be 28 regular teams
    # The two All Star teams (NLS + ALS) are no longer included in these data
    expect_equal(length(roster), 28)

    # In 1995, 39 different players played for TOR
    expect_equal(nrow(roster$TOR), 39)

})

test_that("Game downloading works", {

    skip_if_offline(host = "retrosheet.org")

    game <- get_retrosheet("game", 2012, cache = "testdata")

    # Confirm the returned dataframe has the correct dimensions
    expect_equal(dim(game), c(2430, 161))

})

test_that("Play downloading works", {

    skip_if_offline(host = "retrosheet.org")

    play <- get_retrosheet("play", 2012, "SFN", cache = "testdata")

    # There were 81 different SFN home games in 2012
    expect_equal(length(play), 81)

    # There were 68 different plays in the first game of the year
    expect_equal(nrow(play[[1]]$play), 68)

    # There were 4 different substitutions in the first game of the year
    expect_equal(nrow(play[[1]]$sub), 4)

    # There were 18 different starters in the first game of the year
    # (because it's a pre-DL NL game, there are always 18 different starters)
    expect_equal(nrow(play[[1]]$start), 18)

    # There are 26 different bits of "info" associated with that game00
    expect_equal(nrow(play[[1]]$info), 26)

})

test_that("Data is cleaned up as expected", {

    skip_if_offline(host = "retrosheet.org")

    game <- get_retrosheet("game", 2012, cache = "testdata")

    expect_true("data.frame" %in% class(game))
    expect_equal(class(game$Date), "Date")  # Dates are correct data type
    expect_false(any(game$Completion == "", na.rm = TRUE))  # Confirm that empty strings are converted to NAs
    expect_false(any(game$Forfeit == "", na.rm = TRUE))  # Confirm that empty strings are converted to NAs
    expect_false(any(game$Protest == "", na.rm = TRUE))  # Confirm that empty strings are converted to NAs

    schedule <- get_retrosheet(type = "schedule", year = 1995, cache = "testdata")
    expect_true("data.frame" %in% class(schedule))
    expect_equal(class(schedule$Date), "Date")  # Confirm that dates are Dates
    expect_equal(class(schedule$GameNo), "integer")  # Confirm that simple data (esp game numbers) are integers

    # Warn if trying to use stringsAsFactors
    expect_warning(
        get_retrosheet("schedule", 1995, schedSplit = "TimeOfDay", stringsAsFactors = TRUE, cache = "testdata")
        )
})

# Delete any previously cached data
unlink("testdata", recursive = TRUE)

.onAttach <- function(libname, pkgname) {
    packageStartupMessage("\nFor Retrosheet data obtained with this package:\n
The information used here was obtained free of charge from
and is copyrighted by Retrosheet. Interested parties may
contact Retrosheet at \"www.retrosheet.org\"\n")
}

#'
#' Import single-season retrosheet data as a structured R object
#'
#' This function downloads and parses data from \url{https://www.retrosheet.org}
#' for the game-log, event, (play-by-play), roster, and schedule files.
#'
#' @param type character.  This argument can take on either of "game" for
#' game-logs, "play" for play-by-play (a.k.a. event) data, "roster" for
#' team rosters, or "schedule" for the game schedule for the given year.
#' @param year integer. A valid four-digit year.
#' @param team character. Only to be used if \code{type = "play"}.
#' A single valid team ID for the given year. For available team IDs for the
#' given year call \code{getTeamIDs(year)}.  The available teams
#' are in the "TeamID" column.
#' @param schedSplit One of "Date", "HmTeam", or "TimeOfDay" to return a list
#' split by the given value, or NULL (the default) for no splitting.
#' @param stringsAsFactors logical. The \code{stringsAsFactors} argument as
#' used in \code{\link[base]{data.frame}}. Currently applicable to types "game" and "schedule".
#' @param cache character. Path to local cache of retrosheet data. If file doesn't exist,
#' files will be saved locally for future use. Defaults to "NA" so as not to save local data without
#' explicit permission
#' @return The following return values are possible for the given \code{type}
#' \itemize{
#' \item \code{game} - a data frame of gamelog data for the given year
#' \item \code{play} - a list, each element of which is a single game's play-by-play
#' data for the given team and year.  Each list element is also a list, containing
#' the play-by-play data split into individual matrices.
#' \item \code{roster} - a named list, each element containing the roster
#' for the named team for the given year, as a data frame.
#' \item \code{schedule} - a data frame containing the game schedule for the given year
#' }
#'
#' @examples
#' \donttest{
#' ## get the full 1995 season schedule
#' getRetrosheet("schedule", 1995)
#'
#' ## get the same schedule, split by time of day
#' getRetrosheet("schedule", 1995, schedSplit = "TimeOfDay")
#'
#' ## get the roster data for the 1995 season, listed by team
#' getRetrosheet("roster", 1995)
#'
#' ## get the full gamelog data for the 2012 season
#' getRetrosheet("game", 2012)
#'
#' ## get the play-by-play data for the San Francisco Giants' 2012 season
#' getRetrosheet("play", 2012, "SFN")
#' }
#'
#' @importFrom httr http_error RETRY write_disk timeout
#' @importFrom stringi stri_split_fixed
#' @importFrom stats setNames
#' @importFrom utils read.csv unzip
#' @export

getRetrosheet <- function(type, year, team, schedSplit = NULL, stringsAsFactors = FALSE, cache = NA) {

    type <- match.arg(type, c("game", "play", "roster", "schedule"))

    if(type == "play" && missing(team)) {
        stop("argument 'team' must be supplied when 'type = \"play\"")
    }

    path <- switch(type,
                   "game" = "/gamelogs/gl%d.zip",
                   "play" = "/events/%deve.zip",
                   "roster" = "/events/%deve.zip",
                   "schedule" = "/schedule/%dSKED.zip")

    # If cache is NA, download to a temp location
    if (is.na(cache)) {
        fullPath <- sprintf(paste0("https://www.retrosheet.org", path), year)
        if(!http_error(fullPath)) {
            tmp <- tempfile()
            on.exit(unlink(tmp))
            RETRY("GET", url = fullPath, write_disk(tmp, overwrite=TRUE), timeout(15))
        } else {
            stop(sprintf("'%s' is not a valid url or path", fullPath))
        }
    } else {
        # If the cache is something, then:

        # Drop trailing slash on cache path
        if (substring(cache, nchar(cache)) == "/") {
            cache <- substring(cache, 1, nchar(cache) - 1)
        }
        fullPath <- sprintf(paste0(cache, path), year)
        remotePath <- sprintf(paste0("https://www.retrosheet.org", path), year)
        # If the file doesn't exist, download it
        if (!file.exists(fullPath)) {
            # Make the folder if it doesn't exist
            if (!dir.exists(dirname(fullPath))) {
                dir.create(dirname(fullPath), recursive = TRUE)
            }
            # Download the file to the folder
            message("Caching to: ", fullPath)
            RETRY("GET", url = remotePath, write_disk(fullPath, overwrite=TRUE), timeout(15))
        } else {
            message("Using local cache: ", fullPath)
        }
        # Then use the locally downloaded file as the data source
        tmp <- fullPath
    }

    fname <- unzip(tmp, list = TRUE)$Name

    if (type == "schedule") {

        # Open the file, get the first row that contains the original names
        zcon <- unz(tmp, filename = fname)
        unparsed_file <- readLines(zcon)
        close(zcon)

        original_cols <- unlist(strsplit(unparsed_file[[1]], ","))

        # Read the unparsed CSV and use the correct column names
        out <- read.csv(
            text = paste(unparsed_file, collapse = "\n"),
            header = FALSE, skip = 1L,
            col.names = get_column_names(original_cols, type = "schedule"),
            stringsAsFactors = stringsAsFactors
        )

        out <- out[!is.na(out$GameNo), ] # Filter out the last line, which is often parsed incorrectly
        if(is.character(schedSplit)) {
            schedSplit <- match.arg(schedSplit, c("Date", "HmTeam", "TimeOfDay"))
            out <- split(out, out[[schedSplit]])
        }
        return(out)
    }

    if (type == "game") {

        # Open the file, get the first row that contains the original names
        zcon <- unz(tmp, filename = fname)
        unparsed_file <- readLines(zcon)
        close(zcon)

        original_cols <- unlist(strsplit(unparsed_file[[1]], ","))

        # Read the unparsed CSV and use the correct column names
        out <- read.csv(
            text = paste(unparsed_file, collapse = "\n"),
            header = FALSE,
            col.names = get_column_names(original_cols, type = type),
            stringsAsFactors = stringsAsFactors
        )
        return(out)
    }

    if (type == "roster") {

        rosFiles <- grep(".ROS", fname, value = TRUE, fixed = TRUE)

        read <- lapply(rosFiles, function(x) {

            zcon <- unz(tmp, filename = x)
            unparsed_file <- strsplit(readLines(zcon), ",")
            close(zcon)

            original_cols <- unlist(strsplit(unparsed_file[[1]], ","))

            o <- read.csv(
                text = paste(unparsed_file, collapse = "\n"),
                header = FALSE,
                col.names = get_column_names(original_cols, type = type),
                stringsAsFactors = stringsAsFactors
            )
            o
        })
        out <- setNames(read, substr(rosFiles, 1L, 3L))
        return(out)
    }
    zcon <- unz(tmp, filename = paste0("TEAM", year))
    allTeams <- readLines(zcon)
    close(zcon)
    team <- match.arg(team, substr(allTeams, 1L, 3L))

    # function for single game parsing
    doGame <- function(x) {
        sc <- scan(text = x, sep = ",", what = "", flush = TRUE, quiet = TRUE)
        outer <- retrosheetFields$eventOuter
        v <- vector("list", 8L)
        for(i in seq_len(8L)) {
            sx <- substr(x, regexpr(",", x, fixed = TRUE) + 1L, nchar(x))
            v[[i]] <- sx[match(sc, outer) == i]
        }
        names(v) <- outer
        v[-c(1:2, 6L)] <- lapply(v[-c(1:2, 6L)], stri_split_fixed, ",", simplify = TRUE)

        ans <- Map(function(A, B) {
            if (NCOL(A) == length(B) | is.null(dim(A))) {
                colnames(A) <- B; A
            } else { NULL }
        },  A = v, B = retrosheetFields$eventInner)
        ans
    }

    rgx <- paste(team, "EV", sep = ".")
    fnm <- grep(rgx, fname, value = TRUE, fixed = TRUE)
    zcon <- unz(tmp, filename = fnm)
    r <- readLines(zcon)
    close(zcon)
    g <- grepl("^id", r)
    sr <- unname(split(gsub("\"", "", r), cumsum(g)))
    res <- lapply(sr, doGame)
    res
}

test_that("Caching works", {

    skip_if_offline(host = "retrosheet.org")

    # Delete any previously cached data
    unlink("testdata", recursive = TRUE)

    # With caching
    schedule_1 <- getRetrosheet("schedule", 1995, cache = "testdata")
    schedule_1a <- getRetrosheet("schedule", 1995, cache = "testdata/") # Test with trailing slash
    roster_1 <- getRetrosheet("roster", 1995, cache = "testdata")
    game_1 <- getRetrosheet("game", 2012, cache = "testdata")
    play_1 <- getRetrosheet("play", 2012, "SFN", cache = "testdata")

    # Without caching
    schedule_2 <- getRetrosheet("schedule", 1995)
    roster_2 <- getRetrosheet("roster", 1995)
    game_2 <- getRetrosheet("game", 2012)
    play_2 <- getRetrosheet("play", 2012, "SFN")

    expect_equal(schedule_1, schedule_2)
    expect_equal(schedule_1, schedule_1a)
    expect_equal(roster_1, roster_2)
    expect_equal(game_1, game_2)
    expect_equal(play_1, play_2)

    # Re-using cached data should give a message about using a local cache
    expect_message(getRetrosheet("schedule", 1995, cache = "testdata"), "Using local cache: testdata/schedule/1995SKED.zip")

})

test_that("Schedule downloading works", {

    skip_if_offline(host = "retrosheet.org")

    schedule <- getRetrosheet("schedule", 1995, cache = "testdata")
    schedule_splits <- getRetrosheet("schedule", 1995, schedSplit = "TimeOfDay")

    # In 1995, there were 28 teams, each team played a 144 game schedule
    expect_equal(nrow(schedule), 28 * 144 / 2)

    # There were 601 day games, 60 evening games, and 1355 night games
    expect_equal(sapply(schedule_splits, nrow), c(D = 601L, E = 60L, N = 1355L))

})

test_that("Roster downloading works", {

    skip_if_offline(host = "retrosheet.org")

    roster <- getRetrosheet("roster", 1995, cache = "testdata")

    # In 1995, there should be 28 regular teams
    # The two All Star teams (NLS + ALS) are no longer included in these data
    expect_equal(length(roster), 28)

    # Toronto has 39 players on their roster
    expect_equal(nrow(roster$TOR), 39)

})

test_that("Game downloading works", {

    skip_if_offline(host = "retrosheet.org")

    game <- getRetrosheet("game", 2012, cache = "testdata")

    # Each "game" has 161 different attributes
    expect_equal(length(game), 161)

    # In 2012, there were 30 teams, each played 162 games (and each game has two teams)
    expect_equal(nrow(game), 30 * 162 / 2)

})

test_that("Play downloading works", {

    skip_if_offline(host = "retrosheet.org")

    play <- getRetrosheet("play", 2012, "SFN", cache = "testdata")

    # SFN played 81 games in 2012
    expect_equal(length(play), 81)

    # In the first game of the year, there were...
    expect_equal(nrow(play[[1]]$play), 68)     # 68 plays
    expect_equal(nrow(play[[1]]$sub), 4)       # 4 substitutions
    expect_equal(nrow(play[[1]]$start), 2 * 9) # 18 starters

})

test_that("Schedule parses pre-2024 format correctly", {

    skip_if_offline(host = "retrosheet.org")

    schedule <- getRetrosheet("schedule", 1995)

    expect_equal(ncol(schedule), 12)
    expect_false("Location" %in% names(schedule))

})

test_that("Schedule parses 2024+ format correctly", {

    skip_if_offline(host = "retrosheet.org")

    schedule <- getRetrosheet("schedule", 2025)

    expect_equal(ncol(schedule), 13)
    expect_true("Location" %in% names(schedule))
    # 2025 season opened in Tokyo — some games should have a non-empty Location
    expect_true(any(nchar(trimws(schedule$Location)) > 0, na.rm = TRUE))

})

test_that("Schedule loads for most recent completed season without error", {

    skip_if_offline(host = "retrosheet.org")

    most_recent_year <- as.integer(format(Sys.Date(), "%Y")) - 1
    schedule <- getRetrosheet("schedule", most_recent_year)

    expect_true(is.data.frame(schedule))
    expect_gt(nrow(schedule), 0)
    expect_true(all(c("Date", "HmTeam", "VisTeam", "GameNo") %in% names(schedule)))

})

# Delete any previously cached data
unlink("testdata", recursive = TRUE)

#'
#' Partial parser for game-log files
#'
#' Instead of returning the entire file, this function allows the user
#' to choose the columns and date for game-log data.
#'
#' @param year A single four-digit year.
#' @param glFields character.  The desired game-log columns.  This should be a
#' subset of \code{gamelogFields}, and \strong{not} the entire vector.
#' @param date One of either NULL (the default), or a single four-digit
#' character string identifying the date 'mmdd'
#'
#' @importFrom httr RETRY write_disk timeout
#'
#' @export
#'
#' @return
#' \itemize{
#' \item \code{getPartialGamelog} - A data table with dimensions \code{length(date)} x \code{length(glFields)} if  \code{date} is not NULL, otherwise the row dimension is the number of games for the given year.
#' \item \code{gamelogFields} - A character vector of possible values to choose from for the \code{glFlields} argument in \code{getPartialGamelog}.
#' }
#'
#' @examples ## Get Homerun and RBI info for the 2012 season, with park ID
#' \donttest{
#' f <- grep("HR|RBI|Park", gamelogFields, value = TRUE)
#' getPartialGamelog(2012, glFields = f)
#'
#' ## Get Homerun and RBI info for August 25, 2012 - with park ID
#' getPartialGamelog(glFields=f, date = "20120825")
#' }
#'
getPartialGamelog <- function(year, glFields, date = NULL) {

    ## check 'glFields' against package variable 'retrosheetFields$game'
    if(identical(glFields, retrosheetFields$game)) {
        stop(shQuote("getPartialGamelog"), " is for efficiently returning a small subset of the entire file. For the full table, use ", shQuote("getRetrosheet(\"game\", year)"))
    }

    if(missing(year)) {
        year <- substr(date, 1L, 4L)
    }

    ## define the url
    u <- "https://www.retrosheet.org"
    full <- sprintf("%s/gamelogs/gl%s.zip", u, year)

    ## download the file
    tmp <- tempfile()
    on.exit(unlink(tmp))
    RETRY("GET", url = full, write_disk(tmp, overwrite=TRUE), timeout(15))

    ## extract the text file
    fname <- unzip(tmp, list = TRUE)$Name
    unzip(tmp, files = fname)
    on.exit(unlink(fname), add = TRUE)

    ## match 'glFields' against the internal name vector
    sel <- union(1L, sort(match(glFields, retrosheetFields$game)))

    ## read the data
    if(is.null(date)) {

        #out <- fread(fname, select = sel, header = FALSE)
        out <- read.csv(fname, header = FALSE)
        out <- out[sel]

    } else if(is.character(date)) {

        ## get the first column only - this is the 'Date' column
        sc <- scan(fname, sep = ",", flush = TRUE, what = "", quote = "\"", quiet = TRUE)

        ## find rows of matched dates
        if(!length(wh <- which(sc == date))) {
            stop("invalid 'date' given")
        }

        ## read the file - selecting specified date and columns
        readin <- read.csv(fname, header = FALSE)
        out <- readin[readin$V1 == date, sel]
    }

    ## set the names
    names(out) <- retrosheetFields$game[sel]
    ## return the table
    out
}

#' @rdname getPartialGamelog
#'
#' @name gamelogFields
#'
#' @export
#'
gamelogFields <- c("Date", "DblHdr", "Day", "VisTm", "VisTmLg", "VisTmGNum", "HmTm",
    "HmTmLg", "HmTmGNum", "VisRuns", "HmRuns", "NumOuts", "DayNight",
    "Completion", "Forfeit", "Protest", "ParkID", "Attendance", "Duration",
    "VisLine", "HmLine", "VisAB", "VisH", "VisD", "VisT", "VisHR",
    "VisRBI", "VisSH", "VisSF", "VisHBP", "VisBB", "VisIBB", "VisK",
    "VisSB", "VisCS", "VisGDP", "VisCI", "VisLOB", "VisPs", "VisER",
    "VisTER", "VisWP", "VisBalks", "VisPO", "VisA", "VisE", "VisPassed",
    "VisDB", "VisTP", "HmAB", "HmH", "HmD", "HmT", "HmHR", "HmRBI",
    "HmSH", "HmSF", "HmHBP", "HmBB", "HmIBB", "HmK", "HmSB", "HmCS",
    "HmGDP", "HmCI", "HmLOB", "HmPs", "HmER", "HmTER", "HmWP", "HmBalks",
    "HmPO", "HmA", "HmE", "HmPass", "HmDB", "HmTP", "UmpHID", "UmpHNm",
    "Ump1BID", "Ump1BNm", "Ump2BID", "Ump2BNm", "Ump3BID", "Ump3BNm",
    "UmpLFID", "UmpLFNm", "UmpRFID", "UmpRFNm", "VisMgrID", "VisMgrNm",
    "HmMgrID", "HmMgrNm", "WinPID", "WinPNm", "PID", "PNAme", "SavePID",
    "SavePNm", "GWinRBIID", "GWinRBINm", "VisStPchID", "VisStPchNm",
    "HmStPchID", "HmStPchNm", "VisBat1ID", "VisBat1Nm", "VisBat1Pos",
    "VisBat2ID", "VisBat2Nm", "VisBat2Pos", "VisBat3ID", "VisBat3Nm",
    "VisBat3Pos", "VisBat4ID", "VisBat4Nm", "VisBat4Pos", "VisBat5ID",
    "VisBat5Nm", "VisBat5Pos", "VisBat6ID", "VisBat6Nm", "VisBat6Pos",
    "VisBat7ID", "VisBat7Nm", "VisBat7Pos", "VisBat8ID", "VisBat8Nm",
    "VisBat8Pos", "VisBat9ID", "VisBat9Nm", "VisBat9Pos", "HmBat1ID",
    "HmBat1Nm", "HmBat1Pos", "HmBat2ID", "HmBat2Nm", "HmBat2Pos",
    "HmBat3ID", "HmBat3Nm", "HmBat3Pos", "HmBat4ID", "HmBat4Nm",
    "HmBat4Pos", "HmBat5ID", "HmBat5Nm", "HmBat5Pos", "HmBat6ID",
    "HmBat6Nm", "HmBat6Pos", "HmBat7ID", "HmBat7Nm", "HmBat7Pos",
    "HmBat8ID", "HmBat8Nm", "HmBat8Pos", "HmBat9ID", "HmBat9Nm",
    "HmBat9Pos", "Additional", "Acquisition")

test_that("getPartialGamelog examples all work", {

    skip_if_offline(host = "retrosheet.org")

    interesting_columns <- c("ParkID",
                             "VisHR", "VisRBI",
                             "HmHR", "HmRBI")

    example_1 <- getPartialGamelog(2012, glFields = interesting_columns)
    example_2 <- getPartialGamelog(glFields = interesting_columns, date = "20120825")

    # Expect a row for each of the 30 teams playing 81 home games
    expect_equal(nrow(example_1), 30 * 81)

    # There should be six columns - the date, and the five interesting_columns
    expect_equal(ncol(example_1), 1 + length(interesting_columns))

    # There were 15 games played on Aug 25, 2012
    expect_equal(nrow(example_2), 15)

    # There should be six columns - the date, and the five interesting_columns
    expect_equal(ncol(example_2), 1 + length(interesting_columns))
})

#'
#' Files currently available for download
#'
#' A convenience function, returning the base file names of the
#' available downloads for the \code{year} and \code{type} arguments
#' in \code{getRetrosheet}.
#'
#' @return A named list of available single-season Retrosheet event and
#' game-log zip files, and schedule text files. These file names are
#' not intended to be passed to \code{getRetrosheet}, but is simply a
#' fast way to determine if the desired data is available.
#'
#' @examples
#' \donttest{
#' getFileNames()
#' }
#' @importFrom httr content RETRY timeout
#' @importFrom xml2 read_html
#' @importFrom rvest html_attr html_nodes
#' @importFrom stringr str_extract
#'
#' @export

getFileNames <- function() {
    paths <- c(event = "game.htm",
               gamelog = "gamelogs/index.html",
               schedule = "schedule/index.html")

    full <- paste0("https://www.retrosheet.org/", paths)

    docs <- lapply(full, function(x) {
        content <- RETRY("GET", url = x, timeout(15))
        read_html(content, asText = TRUE)
    })

    get_links <- function(x) html_attr(html_nodes(x, "a"), "href")
    links <- lapply(docs, get_links)

    trim_path <- function(x) {
        out <- str_extract(x, pattern = "(gl[0-9]{4}|[0-9]{4}SKED|[0-9]{4}eve)(.zip|.ZIP)")
        out[!is.na(out)]
    }

    trimmed <- lapply(links, trim_path)
    names(trimmed) <- names(paths)
    trimmed
}

test_that("getFileNames() returns the right result", {

    skip_if_offline(host = "retrosheet.org")

    result <- getFileNames()

    # There are at least three different types of files available
    expect_in(c("event", "gamelog", "schedule"), names(result))

    # Event files start in 1913, so there should be at least 2022 - 1913 files
    expect_gte(length(result$event), 2022 - 1913)

    # Game logs go back to 1871, so there should be at least 2022 - 1871 files
    expect_gte(length(result$gamelog), 2022 - 1871)

    # Schedules go back to 1877, so there should be at least 2022 - 1877 files
    expect_gte(length(result$schedule), 2022 - 1877)
})

#'
#' A data frame of ballpark IDs
#'
#' This function returns a two-column data frame of ballpark IDs
#' along with current stadium name
#'
#' @export
#'
#' @examples
#' \donttest{
#' getParkIDs()
#' }
#'

getParkIDs <- function() {

    path <- "https://www.retrosheet.org/parkcode.txt"

    # Make a temp file
    tmp <- tempfile()
    on.exit(unlink(tmp))

    # Get the txt from the path, write to temp file
    RETRY("GET", url = path, write_disk(tmp, overwrite=TRUE), timeout(15))

    nm <- scan(tmp, nlines = 1L, nmax = 2L, what = "", sep = ",", quiet = TRUE)
    what <- setNames(rep_len(list(""), length(nm)), nm)
    scn <- scan(tmp, skip = 1L, what = what, sep = ",", flush = TRUE,
                quote = ",", quiet = TRUE)
    attr(scn, "row.names") <- .set_row_names(length(scn[[1L]]))
    class(scn) <- "data.frame"
    scn

}

#' @name getTeamIDs
#'
#' @title Retrieve team IDs for event files
#'
#' @description This function retrieves the team ID needed for the
#' \code{team} argument of \code{getRetrosheet("play", year, team)}.
#'
#' @param year A single valid four-digit numeric year.
#'
#' @return If the file exists, a named vector of IDs for the given year.
#' Otherwise \code{NA}.
#'
#' @details All currently available years can be retrieved with
#' \code{type.convert(substr(getFileNames()$event, 1L, 4L))}
#'
#' @importFrom httr http_error write_disk RETRY timeout
#' @export
#'
#' @examples \donttest{
#' getTeamIDs(2010)
#' }
#'
getTeamIDs <- function(year) {
    stopifnot(is.numeric(year), length(year) == 1L)
    path <- sprintf("https://www.retrosheet.org/events/%deve.zip", year)
    if (!http_error(path)) {
        tmp <- tempfile()
        on.exit(unlink(tmp))
        RETRY("GET", url = path, write_disk(tmp, overwrite=TRUE), timeout(15))
    } else {
        available <- grep(year, getFileNames()$event)
        if(!length(available)) {
            return(NA)

        }
    }
    fname <- paste0("TEAM", year)
    unzip(tmp, files = fname)
    on.exit(unlink(fname), add = TRUE)

    read <- suppressWarnings(read.csv(fname, header = FALSE, stringsAsFactors = FALSE))[c(1, 4)]

    out <- structure(read[[1L]], .Names = read[[2L]])
    out
}

test_that("getTeamIDs returns the right number of teams", {

    skip_if_offline(host = "retrosheet.org")

    teamids_2012 <- getTeamIDs(2012)
    # In 2012, there were 30 different teams
    # We use `expect_in()` because Retrosheet adds the All Star teams as well
    expect_in(
        c(
            Angels = "ANA", Orioles = "BAL", `Red Sox` = "BOS", `White Sox` = "CHA",
            Indians = "CLE", Tigers = "DET", Royals = "KCA", Twins = "MIN",
            Yankees = "NYA", Athletics = "OAK", Mariners = "SEA", Rays = "TBA",
            Rangers = "TEX", `Blue Jays` = "TOR", Diamondbacks = "ARI", Braves = "ATL",
            Cubs = "CHN", Reds = "CIN", Rockies = "COL", Astros = "HOU",
            Dodgers = "LAN", Marlins = "MIA", Brewers = "MIL", Mets = "NYN",
            Phillies = "PHI", Pirates = "PIT", Padres = "SDN", Giants = "SFN",
            Cardinals = "SLN", Nationals = "WAS"
        ),
        teamids_2012
    )

    # In 1954, there were 16 teams
    # We use `expect_in()` because Retrosheet adds the All Star teams as well
    teamids_1954 <- getTeamIDs(1954)
    expect_in(
        c(
            Orioles = "BAL", `Red Sox` = "BOS", `White Sox` = "CHA", Indians = "CLE",
            Tigers = "DET", Yankees = "NYA", Athletics = "PHA", Senators = "WS1",
            Dodgers = "BRO", Cubs = "CHN", Reds = "CIN", Braves = "MLN",
            Giants = "NY1", Phillies = "PHI", Pirates = "PIT", Cardinals = "SLN"
        ),
        teamids_1954
    )
})

retrosheetFields <- list(

    eventInner = list(
        NULL,
        NULL,
        c("category", "info"),
        c("retroID", "name", "team", "batPos", "fieldPos"),
        c("inning", "team", "retroID", "count", "pitches", "play"),
        NULL,
        c("retroID", "name", "team", "batPos", "fieldPos"),
        c("projCode", "retroID", "ER")
    ),

    eventOuter = c("id", "version", "info", "start", "play", "com", "sub", "data"),

    franchise = c(
        "FranchiseID", "OtherID", "Lg", "Div", "Loc",
        "Nick1", "Nick2", "GmFirst", "GmLast", "City", "State"
    ),

    game = c(
        # Game identification
        "Date", "DblHdr", "Day",
        # Teams
        "VisTm", "VisTmLg", "VisTmGNum", "HmTm", "HmTmLg", "HmTmGNum",
        # Score and game info
        "VisRuns", "HmRuns", "NumOuts", "DayNight", "Completion",
        "Forfeit", "Protest", "ParkID", "Attendance", "Duration",
        # Line scores
        "VisLine", "HmLine",
        # Visitor batting
        "VisAB", "VisH", "VisD", "VisT", "VisHR", "VisRBI",
        "VisSH", "VisSF", "VisHBP", "VisBB", "VisIBB", "VisK",
        "VisSB", "VisCS", "VisGDP", "VisCI", "VisLOB", "VisPs",
        # Visitor pitching
        "VisER", "VisTER", "VisWP", "VisBalks",
        # Visitor fielding
        "VisPO", "VisA", "VisE", "VisPassed", "VisDB", "VisTP",
        # Home batting
        "HmAB", "HmH", "HmD", "HmT", "HmHR", "HmRBI",
        "HmSH", "HmSF", "HmHBP", "HmBB", "HmIBB", "HmK",
        "HmSB", "HmCS", "HmGDP", "HmCI", "HmLOB", "HmPs",
        # Home pitching
        "HmER", "HmTER", "HmWP", "HmBalks",
        # Home fielding
        "HmPO", "HmA", "HmE", "HmPass", "HmDB", "HmTP",
        # Umpires
        "UmpHID", "UmpHNm", "Ump1BID", "Ump1BNm", "Ump2BID", "Ump2BNm",
        "Ump3BID", "Ump3BNm", "UmpLFID", "UmpLFNm", "UmpRFID", "UmpRFNm",
        # Managers
        "VisMgrID", "VisMgrNm", "HmMgrID", "HmMgrNm",
        # Pitchers of record
        "WinPID", "WinPNm", "PID", "PNAme", "SavePID", "SavePNm",
        "GWinRBIID", "GWinRBINm",
        "VisStPchID", "VisStPchNm", "HmStPchID", "HmStPchNm",
        # Visitor batting order
        "VisBat1ID", "VisBat1Nm", "VisBat1Pos",
        "VisBat2ID", "VisBat2Nm", "VisBat2Pos",
        "VisBat3ID", "VisBat3Nm", "VisBat3Pos",
        "VisBat4ID", "VisBat4Nm", "VisBat4Pos",
        "VisBat5ID", "VisBat5Nm", "VisBat5Pos",
        "VisBat6ID", "VisBat6Nm", "VisBat6Pos",
        "VisBat7ID", "VisBat7Nm", "VisBat7Pos",
        "VisBat8ID", "VisBat8Nm", "VisBat8Pos",
        "VisBat9ID", "VisBat9Nm", "VisBat9Pos",
        # Home batting order
        "HmBat1ID", "HmBat1Nm", "HmBat1Pos",
        "HmBat2ID", "HmBat2Nm", "HmBat2Pos",
        "HmBat3ID", "HmBat3Nm", "HmBat3Pos",
        "HmBat4ID", "HmBat4Nm", "HmBat4Pos",
        "HmBat5ID", "HmBat5Nm", "HmBat5Pos",
        "HmBat6ID", "HmBat6Nm", "HmBat6Pos",
        "HmBat7ID", "HmBat7Nm", "HmBat7Pos",
        "HmBat8ID", "HmBat8Nm", "HmBat8Pos",
        "HmBat9ID", "HmBat9Nm", "HmBat9Pos",
        # Misc
        "Additional", "Acquisition"
    ),

    roster = c("retroID", "Last", "First", "Bat", "Throw", "Team", "Pos"),

    schedule = c(
        "Date"      = "Date",
        "Num"       = "GameNo",
        "Day"       = "Day",
        "Visitor"   = "VisTeam",
        "League"    = "VisLg",
        "Game"      = "VisGmNo",
        "Home"      = "HmTeam",
        "League.1"  = "HmLg",
        "Game.1"    = "HmGmNo",
        "Day/Night" = "TimeOfDay",
        "Location"  = "Location",
        "Postponed" = "Postponed",
        "Makeup"    = "Makeup"
    )

)



get_column_names <- function(raw_names, type, field_map = retrosheetFields) {

    # Games don't have column names, so we just return a static vector
    # regardless of the input
    if (type %in% c("game", "roster")) {
        return(field_map[[type]])
    }

    if (type %in% c("schedule")) {
        deduped <- make.unique(raw_names, sep = ".")
        return(
            ifelse(deduped %in% names(field_map[[type]]), field_map[[type]][deduped], deduped)
        )
    }
}

library(rmarkdown)
test_that("Can be used in rmarkdown", {

    skip_if_offline(host = "retrosheet.org")

    # This test fails on systems without recent versions of Pandoc
    # Thus it fails on the MacOS and Solaris CRAN check systems unless we skip this test
    skip_on_cran()
    skip_if_not(rmarkdown::pandoc_available(), "pandoc not available")

    # Borrowed from: https://stackoverflow.com/questions/32310399/testing-code-chunks-of-a-markdown-script

    runAllChunks <- function(rmd, envir=globalenv()){
        tempR <- tempfile(tmpdir = ".", fileext = ".R")
        on.exit(unlink(tempR))
        rmarkdown::render(rmd, output_file = tempR, quiet = TRUE)
    }

    # Test that the file can be run without throwing any errors
    expect_error(
        runAllChunks("test-rmarkdown.Rmd"),
        NA)
})

---
title: "RMarkdown Test File"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(retrosheet)
```

## getTeamIDs and getParkIDS
Here are the baseball teams that were competing in 2012.
```{r getTeamIDs}
getTeamIDs(2012)
```

### Inline getParkIDs
Some of the parks they played in were: `r paste(head(getParkIDs()$NAME, 10), collapse =", ")`

## getPartialGamelog
Here are the games that happened on April Fool's Day, 2018.

```{r get partial gamelog, rows.print = 15}
    getPartialGamelog(glFields=grep("HR|RBI|Park", gamelogFields, value = TRUE), 
                                 date = "20180401")
```


## getRetrosheet
These are the bulky ones, I just cut and paste them from the test file.
```{r getRetrosheet tests}
    schedule_1 <- getRetrosheet("schedule", 1995, cache = "testdata")
    schedule_1a <- getRetrosheet("schedule", 1995, cache = "testdata/") # Test with trailing slash
    roster_1 <- getRetrosheet("roster", 1995, cache = "testdata")
    game_1 <- getRetrosheet("game", 2012, cache = "testdata")
    play_1 <- getRetrosheet("play", 2012, "SFN", cache = "testdata")
```

This is the end of the testing markdown file

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_retrosheet.R
\name{get_retrosheet}
\alias{get_retrosheet}
\title{Import single-season retrosheet data as data frames}
\usage{
get_retrosheet(...)
}
\arguments{
\item{...}{Arguments passed to `getRetrosheet()`. `stringsAsFactors` argument is always FALSE, and will warn if passed as TRUE}
}
\value{
The following return values are possible for the given \code{type}
\itemize{
\item \code{game} - a data frame of gamelog data for the given year
\item \code{play} - a list, each element of which is a single game's play-by-play
data for the given team and year.  Each list element is also a list, containing
the play-by-play data split into individual matrices.
\item \code{roster} - a named list, each element containing the roster
for the named team for the given year, as a data frame.
\item \code{schedule} - a data frame containing the game schedule for the given year
}
}
\description{
This function is a wrapper for getRetrosheet(). It downloads and parses data from
\url{https://www.retrosheet.org} for the game-log, event, (play-by-play), roster, and schedule files.
While getRetrosheet() returns a list of matrices, this function returns an equivalent list of dataframes.
It takes the same arguments, and can act as a drop-in replacement.
}
\examples{
\donttest{
## get the full 1995 season schedule
get_retrosheet("schedule", 1995)

## get the same schedule, split by time of day
get_retrosheet("schedule", 1995, schedSplit = "TimeOfDay")

## get the roster data for the 1995 season, listed by team
get_retrosheet("roster", 1995)

## get the full gamelog data for the 2012 season
get_retrosheet("game", 2012)

## get the play-by-play data for the San Francisco Giants' 2012 season
get_retrosheet("play", 2012, "SFN")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getParkIDs.R
\name{getParkIDs}
\alias{getParkIDs}
\title{A data frame of ballpark IDs}
\usage{
getParkIDs()
}
\description{
This function returns a two-column data frame of ballpark IDs
along with current stadium name
}
\examples{
\donttest{
getParkIDs()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTeamIDs.R
\name{getTeamIDs}
\alias{getTeamIDs}
\title{Retrieve team IDs for event files}
\usage{
getTeamIDs(year)
}
\arguments{
\item{year}{A single valid four-digit numeric year.}
}
\value{
If the file exists, a named vector of IDs for the given year.
Otherwise \code{NA}.
}
\description{
This function retrieves the team ID needed for the
\code{team} argument of \code{getRetrosheet("play", year, team)}.
}
\details{
All currently available years can be retrieved with
\code{type.convert(substr(getFileNames()$event, 1L, 4L))}
}
\examples{
\donttest{
getTeamIDs(2010)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRetrosheet.R
\name{getRetrosheet}
\alias{getRetrosheet}
\title{Import single-season retrosheet data as a structured R object}
\usage{
getRetrosheet(
  type,
  year,
  team,
  schedSplit = NULL,
  stringsAsFactors = FALSE,
  cache = NA
)
}
\arguments{
\item{type}{character.  This argument can take on either of "game" for
game-logs, "play" for play-by-play (a.k.a. event) data, "roster" for
team rosters, or "schedule" for the game schedule for the given year.}

\item{year}{integer. A valid four-digit year.}

\item{team}{character. Only to be used if \code{type = "play"}.
A single valid team ID for the given year. For available team IDs for the
given year call \code{getTeamIDs(year)}.  The available teams
are in the "TeamID" column.}

\item{schedSplit}{One of "Date", "HmTeam", or "TimeOfDay" to return a list
split by the given value, or NULL (the default) for no splitting.}

\item{stringsAsFactors}{logical. The \code{stringsAsFactors} argument as
used in \code{\link[base]{data.frame}}. Currently applicable to types "game" and "schedule".}

\item{cache}{character. Path to local cache of retrosheet data. If file doesn't exist,
files will be saved locally for future use. Defaults to "NA" so as not to save local data without
explicit permission}
}
\value{
The following return values are possible for the given \code{type}
\itemize{
\item \code{game} - a data frame of gamelog data for the given year
\item \code{play} - a list, each element of which is a single game's play-by-play
data for the given team and year.  Each list element is also a list, containing
the play-by-play data split into individual matrices.
\item \code{roster} - a named list, each element containing the roster
for the named team for the given year, as a data frame.
\item \code{schedule} - a data frame containing the game schedule for the given year
}
}
\description{
This function downloads and parses data from \url{https://www.retrosheet.org}
for the game-log, event, (play-by-play), roster, and schedule files.
}
\examples{
\donttest{
## get the full 1995 season schedule
getRetrosheet("schedule", 1995)

## get the same schedule, split by time of day
getRetrosheet("schedule", 1995, schedSplit = "TimeOfDay")

## get the roster data for the 1995 season, listed by team
getRetrosheet("roster", 1995)

## get the full gamelog data for the 2012 season
getRetrosheet("game", 2012)

## get the play-by-play data for the San Francisco Giants' 2012 season
getRetrosheet("play", 2012, "SFN")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFileNames.R
\name{getFileNames}
\alias{getFileNames}
\title{Files currently available for download}
\usage{
getFileNames()
}
\value{
A named list of available single-season Retrosheet event and
game-log zip files, and schedule text files. These file names are
not intended to be passed to \code{getRetrosheet}, but is simply a
fast way to determine if the desired data is available.
}
\description{
A convenience function, returning the base file names of the
available downloads for the \code{year} and \code{type} arguments
in \code{getRetrosheet}.
}
\examples{
\donttest{
getFileNames()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPartialGamelog.R
\docType{data}
\name{getPartialGamelog}
\alias{getPartialGamelog}
\alias{gamelogFields}
\title{Partial parser for game-log files}
\format{
An object of class \code{character} of length 161.
}
\usage{
getPartialGamelog(year, glFields, date = NULL)

gamelogFields
}
\arguments{
\item{year}{A single four-digit year.}

\item{glFields}{character.  The desired game-log columns.  This should be a
subset of \code{gamelogFields}, and \strong{not} the entire vector.}

\item{date}{One of either NULL (the default), or a single four-digit
character string identifying the date 'mmdd'}
}
\value{
\itemize{
\item \code{getPartialGamelog} - A data table with dimensions \code{length(date)} x \code{length(glFields)} if  \code{date} is not NULL, otherwise the row dimension is the number of games for the given year.
\item \code{gamelogFields} - A character vector of possible values to choose from for the \code{glFlields} argument in \code{getPartialGamelog}.
}
}
\description{
Instead of returning the entire file, this function allows the user
to choose the columns and date for game-log data.
}
\examples{
## Get Homerun and RBI info for the 2012 season, with park ID
\donttest{
f <- grep("HR|RBI|Park", gamelogFields, value = TRUE)
getPartialGamelog(2012, glFields = f)

## Get Homerun and RBI info for August 25, 2012 - with park ID
getPartialGamelog(glFields=f, date = "20120825")
}

}
\keyword{datasets}
